---------------------------------------------------
---------------------------------------------------
--  Easy 1 Processor Architecture				 --
--  University of Puerto Rico					 --
--  Mayaguez Campus								 --
--	Department of Computer Engineering			 --
-- 												 --
--	Copyright (c) 2003							 --
--	All rigths reserved.						 --
--										   		 --
--	Prof. Bienvenido Vlez						 --
--	Abdiel Avils								 --
--	Rafael S. Garca							 --
--	Nesmary Hernndez 							 --
-- 	Carlos Prez								 --
---------------------------------------------------
---------------------------------------------------
-- 	[easy1_a0.vhd]
--	Easy 1 Address Register
--	This module describes the Easy 1 Address Register.
--	The Easy 1 address register is composed of a 
--	16 bits register that holds the address for the
--	current instruction and one multiplexer.
---------------------------------------------------	
---------------------------------------------------
--	Libraries declarations
library ieee;
use ieee.std_logic_1164.all; 
use easy1_package.all;
--
---------------------------------------------------
--	Entity declaration
--	@clock		- clock source
--	@a0_sel    	- selection signal for the mux.
--				  This signal selects the address 
--				  source from the A bus or the PC.
--	@reset     	- reset signal
--	@a0_le 		- signal to load the register with the 
--				  next instruction address.
--	@pc_bus   	- input from the Program Counter
--	@a_bus     	- A Bus
--	@a0_out		- output of the address register to 
--				  the Address Bus.
--												  
entity easy1_a0 is
	port(
		clock 	: in std_logic:='0';
		a0_sel	: in std_logic:='0';
		reset   : in std_logic:='0';
		a0_le	: in std_logic:='0';
		pc_bus	: in std_logic_vector(15 downto 0):=X"0000";
		a_bus	: in std_logic_vector(15 downto 0):=X"0000";
		a0_out	: out std_logic_vector(15 downto 0):=X"0000"
		);
end easy1_a0;
--
----------------------------------------------------
--	Architecture declaration
--	Signals:
--		@tmp_input 		- signal that connects the ouput
--						  from the multiplexer to the 
--						  register input.
--
--	Components :
--		@easy1_reg_ms	- master/slave register
--
architecture arq of easy1_a0 is

	component easy1_reg_ms
		port(
			clock	:	in std_logic:='0';
			input	:	in std_logic_vector(15 downto 0):=X"0000";
			load	:	in std_logic:='0';
			clear	:	in std_logic:='0';
			o		:	out std_logic_vector(15 downto 0):=X"0000"
			);
	end component;
	
	signal tmp_input	: std_logic_vector(15 downto 0):=X"0000";
	
begin							   
	--	Port mapping:
	--		Component instances are mapped to internal signals,
	--		inputs, and outputs.
	a0: easy1_reg_ms
	port map(
		clock => clock,
		input => tmp_input,
		load => a0_le,
		clear => reset,
		o => a0_out
		);

	--	Sensitivity List :
	--		This list holds the signals that modify and 
	--		control the multiplexer and the register.
	process(a0_sel,pc_bus,a_bus)
	begin
		case a0_sel is
			--	The input to the address register comes
			--	from the Program Counter.
			when '0' =>
			tmp_input <= pc_bus after DELAY_1_LEVEL;
			--	The input to the address register comes
			--	from the A Bus.
			when '1' =>
			tmp_input <= a_bus after DELAY_1_LEVEL;
			when others => null;	   
		end case;
	end process;
	
end arq;
----------------------------------------------------
----------------------------------------------------